`timescale 1ns/1ns

module M508Emu(
   i_extrst,
   i_extclk,

   i_rom_dat,
   o_rom_adr,
   o_rom_nread,
   i_rom_valid,
   i_rom_nbusy,

   o_led_dat,
   o_led_adr,
   o_led_we,

   i_sw,
   o_led_onboard
);
   input i_extrst;
   input i_extclk;

   // Data Rom Interface
   input  [15:0] i_rom_dat;
   output [ 8:0] o_rom_adr;
   output        o_rom_nread;
   input         i_rom_valid;
   input         i_rom_nbusy;

   // LED Matrix Controller Interface
   output [15:0] o_led_dat;
   output [ 4:0] o_led_adr;
   output        o_led_we;

   // MicroKIT II Onboard Function Interface
   input  [3:0] i_sw;
   output [7:0] o_led_onboard;
   
   wire Q;
   wire [15:0] cmd;
   wire [8:0] adr;
   wire fosc,cmd_freq,cmd_flash;
   wire reset;
   wire [7:0] wdeb;
   wire wf;
   wire wr_pc;
   wire [8:0] pc_in;
   wire wr_tmr;
   wire [7:0] tim_out;
   wire [7:0] opt;
   wire [7:0] db_out;
   wire wr_wreg;
   wire [7:0] wreg;
   wire [7:0] db_in;
   wire cin,cout,dc,zf,wr_fr,wr_cf,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris;
   wire [7:0] gp_a_in, gp_a_out, tris_a_out;
   wire [7:0] gp_b_in, gp_b_out, tris_b_out;
   wire [7:0] gp_c_in, gp_c_out, tris_c_out;
   wire [7:0] gp_d_in, gp_d_out, tris_d_out;
   wire [7:0] gp_e_in, gp_e_out, tris_e_out;
   wire [7:0] gp_f_in, gp_f_out, tris_f_out;
   wire [7:0] gp_g_in, gp_g_out, tris_g_out;

   assign cmd_flash=cmd_freq | cmd_skip | wr_pc;
   assign reset = i_extrst;
   assign fosc  = i_extclk;
   
   //Power On Reset ==============================================
//	mReset Rst(int_reset);
   //Timing Generator ============================================
   mTmgGen tmgen(reset,fosc,Q);

   //Program Memory Module =======================================
   mProgMem pmem(fosc,reset,adr,cmd,Q,cmd_flash);

   //Command Proceccer ===========================================
   mCmdProc Cproc(cmd[11:0],cmd_freq);

   //Program Counter =============================================
   mPCounter PC(reset,fosc,Q,cmd[11:0],adr,wr_pc,pc_in);

   //Timer 0 =====================================================
   mTimer0 tmr0(
      .reset(reset),
      .clk(fosc),
      .Q(Q),
      .opt(opt[3:0]),
      .wr_timer(wr_tmr),
      .datin(db_out),
      .datout(tim_out),
      .deb()
   );

   //W-Reg =======================================================
   mWreg w_reg(reset,fosc,Q,wr_wreg,db_out,wreg);

   //ALU Module ==================================================
   mALU  alu(cmd[11:0],wreg,db_in,cin,db_out,cout,dc,zf,wr_fr,wr_wreg,wr_cf,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris);

   //File Register ==============================================
   mRegFile regfile(
      .reset(reset),
      .clk(fosc),
      .Q(Q),
      .fadr(cmd[4:0]),
      .datin(db_out),
      .datout(db_in),
      .cin(cout),
      .cout(cin),
      .dc(dc),
      .zf(zf),
      .wr_cf(wr_cf),
      .wr_dc(wr_dc),
      .wr_zf(wr_zf),
      .wr_fr(wr_fr),

      .i_rom_dat(i_rom_dat[15:0]),
      .o_rom_adr(o_rom_adr[8:0]),
      .o_rom_nread(o_rom_nread),
      .i_rom_valid(i_rom_valid),
      .i_rom_nbusy(i_rom_nbusy),
      
      .o_led_dat(o_led_dat[15:0]),
      .o_led_adr(o_led_adr[4:0]),
      .o_led_we(o_led_we),

      .i_sw(i_sw[3:0]),
      .o_led_onboard(o_led_onboard),

      .pc_in(adr),
      .pc_out(pc_in),
      .wr_pc(wr_pc),
      .timer_in(tim_out),
      .wr_tmr(wr_tmr),
      .opt_out(opt),
      .wr_opt(wr_opt)
   );	
   
endmodule
