`timescale 1ns/1ns

//Register Files

//Status Register
module mStatus(reset,clk,Q,wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf,datin,datout);
	input reset,clk;
	input Q;
	input wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] streg;
	
	assign datout=streg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			streg<=8'h18;
		end else if(Q) begin
			if(wr_fr) begin
				streg<=datin;
			end else begin
				if(wr_zf)
					streg[2]<=zf;
				if(wr_dc)
					streg[1]<=dc;
				if(wr_cf)
					streg[0]<=cf;
			end
		end
	end
endmodule

//Memory Structure
module mMemory(reset,clk,Q,write,datin,datout);
   parameter WIDTH = 8;
   parameter INIT  = 8'h00;

   input reset,clk ;
   input Q;
   input write;
   input  [WIDTH-1:0] datin;
   output [WIDTH-1:0] datout;

   reg [WIDTH-1:0] r_reg;
   assign datout=r_reg;

   always@(posedge clk or posedge reset) begin
      if(reset) begin
      	 r_reg <= INIT;
      end
      else if(Q) begin
      	 if(write) begin
      	    r_reg <= datin;
      	 end
      end
      else begin
      	 r_reg <= r_reg;
      end
   end

endmodule

/*
//Option Register
module mOption(reset,clk,Q,wr_opt,datin,datout);
	input reset,clk;
	input Q;
	input wr_opt;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] optreg;
	assign datout=optreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			optreg<=8'hff;
		end else if(Q) begin
			if(wr_opt) begin
				optreg<=datin;
			end
		end
	end
endmodule

//TRIS Register
module mTris(reset,clk,Q,wr_tris,datin,datout);
	input reset,clk;
	input Q;
	input wr_tris;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] trisreg;
	assign datout=trisreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			trisreg<=8'hff;
		end else if(Q) begin
			if(wr_tris) begin
				trisreg<=datin;
			end
		end
	end
endmodule

//FSR Register
module mFsr(reset,clk,Q,wr_fsr,datin,datout);
	input reset,clk;
	input Q;
	input wr_fsr;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] fsrreg;
	assign datout=fsrreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			fsrreg<=8'hff;
		end else if(Q) begin
			if(wr_fsr) begin
				fsrreg<=datin;
			end
		end
	end
endmodule

//Gpio Reg
module mGpio(reset,clk,Q,wr_gpio,datin,datout);
	input reset,clk;
	input Q;
	input wr_gpio;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] gpreg;
	assign datout=gpreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			gpreg<=8'h00;
		end else if(Q) begin
			if(wr_gpio) begin
				gpreg<=datin;
			end
		end
	end
endmodule
*/

module mRegFile(
	reset,clk,Q,	//Reset,fosc,Q state
	fadr,			//File Reg Address
	//ALU Interface
	datin,
	datout,
	cin,cout,dc,zf,wr_cf,wr_dc,wr_zf,wr_fr,
	//GPIO (Data Rom)
	i_rom_dat, o_rom_adr, o_rom_nread, i_rom_valid, i_rom_nbusy,
        //GPIO (Led Matrix Controller)
        o_led_dat, o_led_adr, o_led_we,
        //GPIO (MicroKitII OnBoard Function)
        i_sw, o_led_onboard,
	//Program Counter
	pc_in,pc_out,wr_pc,
	//Timer
	timer_in,wr_tmr,
	//OPtion Reg 
	opt_out,wr_opt
);
	input reset,clk;
	input Q;
	input  [4:0] fadr;
	input  [7:0] datin;
	output [7:0] datout;
	input  cin;
	output cout;
	input dc,zf,wr_cf,wr_dc,wr_zf,wr_fr; 

	input  [15:0] i_rom_dat;
	output [ 8:0] o_rom_adr;
	output        o_rom_nread;
	input         i_rom_valid;
	input         i_rom_nbusy;

        output [15:0] o_led_dat;
	output [4:0]  o_led_adr;
	output        o_led_we;

	input  [3:0]  i_sw;
	output [7:0]  o_led_onboard;

	input  [8:0] pc_in;	//Program Counter
	output [8:0] pc_out;
	output wr_pc;
	input  [7:0] timer_in;
	output wr_tmr;
	output [7:0] opt_out;
	input wr_opt;

	reg  [7:0] ram[14:31];  //RAM

	wire [4:0] iadr;	//internal Address
	wire [7:0] fsr_out;	//FSR Reg
	wire [7:0] st_out;	//Status Reg

	//Write Signals
	wire wr_st,wr_fsr,wr_ram;
	wire wr_rom_adr  ;
	wire wr_rom_ctrl ;
	wire wr_led_dat_l;
	wire wr_led_dat_h;
	wire wr_led_ctrl ;
	wire wr_led_obd  ;

	// RAM (5'h0e <= iadr <= 5'h1f)
	wire   w_ram_adr_sel;
	assign w_ram_adr_sel = (iadr[4]|(iadr[3]&iadr[2]&iadr[1])); 
	
	// IADR mapping
	parameter P_ADR_ROM_DAT_L = 5'h05;
	parameter P_ADR_ROM_DAT_H = 5'h06;
	parameter P_ADR_ROM_ADR   = 5'h07;
	parameter P_ADR_ROM_CTRL  = 5'h08;
	parameter P_ADR_LED_DAT_L = 5'h09;
	parameter P_ADR_LED_DAT_H = 5'h0A;
	parameter P_ADR_LED_CTRL  = 5'h0B;
	parameter P_ADR_SW        = 5'h0C;
	parameter P_ADR_LED_OBD   = 5'h0D;

	// Data Rom Interface
	wire [7:0] w_rom_dat_l;
	wire [7:0] w_rom_dat_h;
	wire [7:0] w_rom_adr;
	wire [7:0] w_rom_ctrl;
	wire       w_rom_valid;
	wire       w_rom_nbusy;
	wire       w_rom_nread;
	wire       w_rom_adr_msb;

	assign o_rom_adr   = {w_rom_adr_msb, w_rom_adr};
	assign o_rom_nread = w_rom_nread;
	assign w_rom_ctrl =
	   {2'b00, w_rom_valid, w_rom_nbusy, 2'b00, w_rom_nread, w_rom_adr_msb} ;

	// LED Matrix Controller Interface
	wire [7:0] w_led_dat_l;
	wire [7:0] w_led_dat_h;
	wire [7:0] w_led_ctrl;
	wire [4:0] w_led_adr;
	wire       w_led_we;

	assign o_led_dat = { w_led_dat_h, w_led_dat_l };
	assign o_led_adr = w_led_adr;
	assign o_led_we  = w_led_we;
	assign w_led_ctrl = { 2'b00, w_led_we, w_led_adr} ;

	// MircoKIT OnBoard Function Interface
	wire [7:0] w_sw;
	wire [7:0] w_led_obd;

	assign o_led_onboard = w_led_obd ; 


	assign cout=st_out[0];	
	assign pc_out={st_out[5],datin};
	assign iadr=(fadr[4:0]==5'b00000) ? fsr_out[4:0] : fadr;
	assign datout=
		(iadr==5'h01) ? timer_in :
		(iadr==5'h02) ? pc_in[7:0] :
		(iadr==5'h03) ? st_out :
		(iadr==5'h04) ? fsr_out :
		(iadr==P_ADR_ROM_DAT_L) ? w_rom_dat_l :
		(iadr==P_ADR_ROM_DAT_H) ? w_rom_dat_h :
		(iadr==P_ADR_ROM_ADR)   ? w_rom_adr   :
		(iadr==P_ADR_ROM_CTRL)  ? w_rom_ctrl  :
		(iadr==P_ADR_LED_DAT_L) ? w_led_dat_l :
		(iadr==P_ADR_LED_DAT_H) ? w_led_dat_h :
		(iadr==P_ADR_LED_CTRL)  ? w_led_ctrl  :
		(iadr==P_ADR_SW)        ? w_sw        :
		(iadr==P_ADR_LED_OBD)   ? w_led_obd   :
		(w_ram_adr_sel) ? ram[iadr] : 8'h00;

	//Decode by SFR Address
	assign wr_tmr   =(iadr==5'h01) ? wr_fr : 1'b0;
	assign wr_pc    =(iadr==5'h02) ? wr_fr : 1'b0;
	assign wr_st    =(iadr==5'h03) ? wr_fr : 1'b0;
	assign wr_fsr   =(iadr==5'h04) ? wr_fr : 1'b0;
//      assign wr_rom_dat_l =(iadr==P_ADR_ROM_DAT_L) ? wr_fr : 1'b0;
//      assign wr_rom_dat_h =(iadr==P_ADR_ROM_DAT_H) ? wr_fr : 1'b0;
	assign wr_rom_adr   =(iadr==P_ADR_ROM_ADR)   ? wr_fr : 1'b0;
	assign wr_rom_ctrl  =(iadr==P_ADR_ROM_CTRL)  ? wr_fr : 1'b0;
	assign wr_led_dat_l =(iadr==P_ADR_LED_DAT_L) ? wr_fr : 1'b0;
	assign wr_led_dat_h =(iadr==P_ADR_LED_DAT_H) ? wr_fr : 1'b0;
	assign wr_led_ctrl  =(iadr==P_ADR_LED_CTRL)  ? wr_fr : 1'b0;
//      assign wr_sw        =(iadr==P_ADR_SW)        ? wr_fr : 1'b0;
	assign wr_led_obd   =(iadr==P_ADR_LED_OBD)   ? wr_fr : 1'b0;
	assign wr_ram=(w_ram_adr_sel) ? wr_fr : 1'b0;
	
	always@(posedge reset or posedge clk) begin
	   if(reset) begin
	         ram[14]<=8'h00;
	         ram[15]<=8'h00;
	         ram[16]<=8'h00;
	         ram[17]<=8'h00;
	         ram[18]<=8'h00;
	         ram[19]<=8'h00;
	         ram[20]<=8'h00;
	         ram[21]<=8'h00;
	         ram[22]<=8'h00;
	         ram[23]<=8'h00;
	         ram[24]<=8'h00;
	         ram[25]<=8'h00;
	         ram[26]<=8'h00;
	         ram[27]<=8'h00;
	         ram[28]<=8'h00;
	         ram[29]<=8'h00;
	         ram[30]<=8'h00;
	         ram[31]<=8'h00;
	   end
	   else if(Q) begin
	      if(wr_ram) begin
	         ram[iadr]<=datin;
   	      end
	   end
	end

	mStatus streg(reset,clk,Q,wr_zf,wr_dc,wr_cf,wr_st,zf,dc,cin,datin,st_out);

	// mOption optreg(reset,clk,Q,wr_opt,datin,opt_out);
        mMemory #(.INIT(8'hff)) optreg(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_opt),
           .datin(datin),
           .datout(opt_out)
        );

	// mFsr    fsrreg(reset,clk,Q,wr_fsr,datin,fsr_out);
        mMemory #(.INIT(8'hff)) fsrreg(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_fsr),
           .datin(datin),
           .datout(fsr_out)
        );

        mMemory #(.WIDTH(16), .INIT(8'h00)) mem_rom_dat(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(1'b1),
           .datin(i_rom_dat),
           .datout({w_rom_dat_h, w_rom_dat_l})
        );

        mMemory #(.WIDTH(8), .INIT(8'h00)) mem_rom_adr(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_rom_adr),
           .datin(datin),
           .datout(w_rom_adr)
        );

        mMemory #(.WIDTH(1), .INIT(1'b0)) mem_rom_valid(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(1'b1),
           .datin (i_rom_valid),
           .datout(w_rom_valid)
        );

        mMemory #(.WIDTH(1), .INIT(1'b0)) mem_rom_nbusy(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(1'b1),
           .datin (i_rom_nbusy),
           .datout(w_rom_nbusy)
        );

        mMemory #(.WIDTH(1), .INIT(1'b1)) mem_rom_nrerad(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_rom_ctrl),
           .datin(datin[1]),
           .datout(w_rom_nread)
        );

        mMemory #(.WIDTH(1), .INIT(1'b0)) mem_rom_adr_msb(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_rom_ctrl),
           .datin(datin[0]),
           .datout(w_rom_adr_msb)
        );

        mMemory #(.WIDTH(8), .INIT(8'h00)) mem_led_dat_h(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_led_dat_h),
           .datin(datin),
           .datout(w_led_dat_h)
        );

        mMemory #(.WIDTH(8), .INIT(8'h00)) mem_led_dat_l(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_led_dat_l),
           .datin(datin),
           .datout(w_led_dat_l)
        );

        mMemory #(.WIDTH(5), .INIT(5'h00)) mem_led_adr(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_led_ctrl),
           .datin(datin[4:0]),
           .datout(w_led_adr)
        );

        mMemory #(.WIDTH(1), .INIT(1'b0)) mem_led_we(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_led_ctrl),
           .datin(datin[5]),
           .datout(w_led_we)
        );

        mMemory #(.WIDTH(4), .INIT(4'h0)) mem_sw(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(1'b1),
           .datin(i_sw),
           .datout(w_sw[3:0])
        );
	assign w_sw[7:4] = 4'b0000;

        mMemory #(.WIDTH(8), .INIT(8'h00)) mem_led_obd(
           .reset(reset),
           .clk(clk),
           .Q(Q),
           .write(wr_led_obd),
           .datin(datin),
           .datout(w_led_obd)
        );

endmodule
