`timescale 1ns/1ns

`include "OperationList.v"

// Variable
`define V_ROM_ADR_L  5'b0_1110
`define V_ROM_ADR_H  5'b0_1111
`define V_STT_ADR_L  5'b1_0000
`define V_STT_ADR_H  5'b1_0001
`define V_LED_ADR    5'b1_0010
`define V_TMR1       5'b1_0011
`define V_TMR1_MAX   5'b1_0100

// Address Label
`define A_TMR0_LOOP  9'h010
`define A_TMR1_LOOP  9'h018
`define A_TMR1_INC   9'h020
`define A_TMR1_CLR   9'h024
`define A_WAIT_TMR0  9'h028
`define A_START_TRS  9'h030
`define A_WT_VALID0  9'h040
`define A_WT_VALID1  9'h048
`define A_TRANS_DAT  9'h050
`define A_UPDT_ADDR  9'h060
`define A_INC_LEDAD  9'h068
`define A_NEXT_SET0  9'h070
`define A_NEXT_SET1  9'h078

`define F_INC_ROMAD  8'hf0
`define F_INC_STTAD  8'hf8



module mFlashMem(adr,dat);
	input  [ 8:0] adr;
	output [11:0] dat;
	reg    [11:0] romdat;
	assign dat=romdat;
	always @(adr) begin
		case (adr)
        9'h000  : romdat={ `OP_MOVLW,       8'h00        } ;  // Initialize
        9'h001  : romdat={ `OP_MOVF,  `F,   `V_ROM_ADR_L } ;
        9'h002  : romdat={ `OP_MOVF,  `F,   `V_ROM_ADR_H } ;
        9'h003  : romdat={ `OP_MOVF,  `F,   `V_STT_ADR_L } ;
        9'h004  : romdat={ `OP_MOVF,  `F,   `V_STT_ADR_H } ;
        9'h005  : romdat={ `OP_MOVF,  `F,   `V_LED_ADR   } ;
        9'h006  : romdat={ `OP_MOVF,  `F,   `V_TMR1      } ;
        9'h007  : romdat={ `OP_MOVLW,       8'h20        } ;
        9'h008  : romdat={ `OP_MOVWF,       `V_TMR1_MAX  } ;
        9'h009  : romdat={ `OP_MOVLW,       8'h07        } ;
        9'h00a  : romdat={ `OP_OPTION                    } ;  // Timer0 1/256
        9'h00b  : romdat={ `OP_GOTO,        `A_TMR0_LOOP } ;
        
        // wait Update Timing
        `A_TMR0_LOOP + 9'h000 : romdat={ `OP_MOVF,  `W,   `TMR0        } ;
        `A_TMR0_LOOP + 9'h001 : romdat={ `OP_BTFSS, 3'd2, `STATUS      } ;  // check STATUS.Z
        `A_TMR0_LOOP + 9'h002 : romdat={ `OP_GOTO,        `A_TMR0_LOOP } ;
        `A_TMR0_LOOP + 9'h003 : romdat={ `OP_MOVLW,       8'h01        } ;
        `A_TMR0_LOOP + 9'h004 : romdat={ `OP_MOVWF,       `TMR0        } ;
        `A_TMR0_LOOP + 9'h005 : romdat={ `OP_GOTO,        `A_TMR1_LOOP } ;  // 

//      `A_TMR1_LOOP + 9'h000 : romdat={ `OP_MOVF,  `W,   `SW          } ;
//      `A_TMR1_LOOP + 9'h001 : romdat={ `OP_MOVWF,       `LEDOBD      } ;
        `A_TMR1_LOOP + 9'h000 : romdat={ `OP_MOVF,  `W,   `V_TMR1_MAX  } ;
        `A_TMR1_LOOP + 9'h001 : romdat={ `OP_XORWF, `W,   `V_TMR1      } ;
        `A_TMR1_LOOP + 9'h002 : romdat={ `OP_BTFSS, 3'd2, `STATUS      } ;  // check (TMR1==TMR1_MAX)
        `A_TMR1_LOOP + 9'h003 : romdat={ `OP_GOTO,        `A_TMR1_INC  } ;  // 
        `A_TMR1_LOOP + 9'h004 : romdat={ `OP_GOTO,        `A_TMR1_CLR  } ;  // 

        `A_TMR1_INC  + 9'h000 : romdat={ `OP_INCF,  `F,   `V_TMR1      } ;
        `A_TMR1_INC  + 9'h001 : romdat={ `OP_GOTO,        `A_WAIT_TMR0 } ;

        `A_TMR1_CLR  + 9'h000 : romdat={ `OP_CLRF,        `V_TMR1      } ;
        `A_TMR1_CLR  + 9'h001 : romdat={ `OP_GOTO,        `A_START_TRS } ;

        `A_WAIT_TMR0 + 9'h000 : romdat={ `OP_BTFSS, 3'd0, `TMR0        } ;  // wait (TMR0 != 0)
        `A_WAIT_TMR0 + 9'h001 : romdat={ `OP_GOTO,        `A_WAIT_TMR0 } ;
        `A_WAIT_TMR0 + 9'h002 : romdat={ `OP_GOTO,        `A_TMR0_LOOP } ;

        // Update LED Matrix Data
        `A_START_TRS + 9'h000 : romdat={ `OP_BTFSS, 3'd4, `ROMCTRL     } ;  // wait nbusy negate
        `A_START_TRS + 9'h001 : romdat={ `OP_GOTO,        `A_START_TRS } ;
        `A_START_TRS + 9'h002 : romdat={ `OP_MOVF,  `W,   `V_ROM_ADR_L } ;
        `A_START_TRS + 9'h003 : romdat={ `OP_MOVWF,       `ROMADR      } ;
        `A_START_TRS + 9'h004 : romdat={ `OP_BTFSC, 3'd0, `V_ROM_ADR_H } ;  // set ROMADR.MSB
        `A_START_TRS + 9'h005 : romdat={ `OP_BSF,   3'd0, `ROMCTRL     } ;  
        `A_START_TRS + 9'h006 : romdat={ `OP_BTFSS, 3'd0, `V_ROM_ADR_H } ;
        `A_START_TRS + 9'h007 : romdat={ `OP_BCF,   3'd0, `ROMCTRL     } ;  
        `A_START_TRS + 9'h008 : romdat={ `OP_BCF,   3'd1, `ROMCTRL     } ;  // assert nread
        `A_START_TRS + 9'h009 : romdat={ `OP_GOTO,        `A_WT_VALID0 } ;

        `A_WT_VALID0 + 9'h000 : romdat={ `OP_BTFSC, 3'd5, `ROMCTRL     } ;  // wait data_valid negate
        `A_WT_VALID0 + 9'h001 : romdat={ `OP_GOTO,        `A_WT_VALID0 } ;
        `A_WT_VALID0 + 9'h002 : romdat={ `OP_GOTO,        `A_WT_VALID1 } ;

        `A_WT_VALID1 + 9'h000 : romdat={ `OP_BTFSS, 3'd5, `ROMCTRL     } ;  // wait data_valid assert
        `A_WT_VALID1 + 9'h001 : romdat={ `OP_GOTO,        `A_WT_VALID1 } ;
        `A_WT_VALID1 + 9'h002 : romdat={ `OP_GOTO,        `A_TRANS_DAT } ;

        `A_TRANS_DAT + 9'h000 : romdat={ `OP_BSF,   3'd1, `ROMCTRL     } ;  // negate nread
        `A_TRANS_DAT + 9'h001 : romdat={ `OP_MOVF,  `W,   `ROMDATL     } ;
        `A_TRANS_DAT + 9'h002 : romdat={ `OP_MOVWF,       `LEDDATL     } ;
        `A_TRANS_DAT + 9'h003 : romdat={ `OP_MOVF,  `W,   `ROMDATH     } ;
        `A_TRANS_DAT + 9'h004 : romdat={ `OP_MOVWF,       `LEDDATH     } ;
        `A_TRANS_DAT + 9'h005 : romdat={ `OP_MOVLW,       8'b0001_1111 } ;
        `A_TRANS_DAT + 9'h006 : romdat={ `OP_ANDWF, `W,   `V_LED_ADR   } ;
        `A_TRANS_DAT + 9'h007 : romdat={ `OP_MOVWF,       `LEDCTRL     } ;
        `A_TRANS_DAT + 9'h008 : romdat={ `OP_BSF,   3'd5, `LEDCTRL     } ;  // assert WE
        `A_TRANS_DAT + 9'h009 : romdat={ `OP_BCF,   3'd5, `LEDCTRL     } ;  // negate WE
        `A_TRANS_DAT + 9'h00a : romdat={ `OP_GOTO,        `A_UPDT_ADDR } ;
        
        `A_UPDT_ADDR + 9'h000 : romdat={ `OP_INCFSZ,`F,   `V_ROM_ADR_L } ;  // ROMADR++
        `A_UPDT_ADDR + 9'h001 : romdat={ `OP_GOTO,        `A_INC_LEDAD } ;
        `A_UPDT_ADDR + 9'h002 : romdat={ `OP_CALL,        `F_INC_ROMAD } ;  // Carry 
        `A_UPDT_ADDR + 9'h003 : romdat={ `OP_GOTO,        `A_INC_LEDAD } ;

        `A_INC_LEDAD + 9'h000 : romdat={ `OP_INCF,  `F,   `V_LED_ADR   } ;  // LEDADR++
        `A_INC_LEDAD + 9'h001 : romdat={ `OP_BTFSS, 3'd5, `V_LED_ADR   } ;  //
        `A_INC_LEDAD + 9'h002 : romdat={ `OP_GOTO,        `A_START_TRS } ;  // continue data transfer
        `A_INC_LEDAD + 9'h003 : romdat={ `OP_GOTO,        `A_NEXT_SET0 } ;  // exit transport loop

        `A_NEXT_SET0 + 9'h000 : romdat={ `OP_CLRF,        `V_LED_ADR   } ;  //
        `A_NEXT_SET0 + 9'h001 : romdat={ `OP_INCFSZ,`F,   `V_STT_ADR_L } ;  // Start ROMADR++
        `A_NEXT_SET0 + 9'h002 : romdat={ `OP_GOTO,        `A_NEXT_SET1 } ;
        `A_NEXT_SET0 + 9'h003 : romdat={ `OP_CALL,        `F_INC_STTAD } ;  // Carry 
        `A_NEXT_SET0 + 9'h004 : romdat={ `OP_GOTO,        `A_NEXT_SET1 } ;

        `A_NEXT_SET1 + 9'h000 : romdat={ `OP_MOVF,  `W,   `V_STT_ADR_L } ;  //
        `A_NEXT_SET1 + 9'h001 : romdat={ `OP_MOVWF,       `V_ROM_ADR_L } ;  //
        `A_NEXT_SET1 + 9'h002 : romdat={ `OP_MOVF,  `W,   `V_STT_ADR_H } ;  //
        `A_NEXT_SET1 + 9'h003 : romdat={ `OP_MOVWF,       `V_ROM_ADR_H } ;  //
        `A_NEXT_SET1 + 9'h004 : romdat={ `OP_GOTO,        `A_TMR0_LOOP } ;  // return to TMR0 loop

        // update ROMADR[8]
  {1'b0,`F_INC_ROMAD}+ 9'h000 : romdat={ `OP_MOVLW,       8'b0000_0001 } ;  // 
  {1'b0,`F_INC_ROMAD}+ 9'h001 : romdat={ `OP_XORWF, `F,   `V_ROM_ADR_H } ;  // 
  {1'b0,`F_INC_ROMAD}+ 9'h002 : romdat={ `OP_RETLW,       8'b0000_0000 } ;  // 

        // update STTADR[8]
  {1'b0,`F_INC_STTAD}+ 9'h000 : romdat={ `OP_MOVLW,       8'b0000_0001 } ;  // 
  {1'b0,`F_INC_STTAD}+ 9'h001 : romdat={ `OP_XORWF, `F,   `V_STT_ADR_H } ;  // 
  {1'b0,`F_INC_STTAD}+ 9'h002 : romdat={ `OP_RETLW,       8'b0000_0000 } ;  // 


        default : romdat=12'h0fff;
		endcase
	end
endmodule

