`timescale 1ns/1ns

module mLcdDisp (
   i_clk,
   o_led,
   i_sw,
   o_row,
   o_col
);
   input         i_clk;   // External Input 50MHz Clock
   output [7:0]  o_led;   // to 8 LEDs on MicroKit (no use)
   input  [3:0]  i_sw;    // from 4 Switchs on MicroKit
   output [15:0] o_row;
   output [31:0] o_col;

   reg r_iclk;

   wire w_led_we;
   wire w_reset;
   wire w_iclk;
   wire [8:0]  w_rom_adr;
   wire [15:0] w_rom_dat;
   wire [4:0]  w_led_adr;
   wire [15:0] w_led_dat;
   wire w_nbusy;
   wire w_nread;
   wire w_rom_dat_valid;
   wire [15:0] w_row;
   wire [31:0] w_col;
   wire [7:0]  w_led_obd;
   
   assign w_reset = (i_sw[3] == 1'b0)? 1'b1: 1'b0;

   assign o_row   = w_row;
   assign o_col   = w_col;
   assign o_led   = ~w_led_obd;


   always @(posedge w_reset or posedge i_clk) begin
      if(w_reset) begin
         r_iclk <= 1'b0;
      end
      else begin
         r_iclk <= ~r_iclk;
      end
   end
   
   assign w_iclk = r_iclk;


   mDataRom DataRom(
	  .addr(w_rom_adr),          // i[8:0]
	  .nread(w_nread),           // i
	  .data_valid(w_rom_dat_valid), // o
	  .dataout(w_rom_dat),           // o[15:0]
	  .nbusy(w_nbusy)                // o
   );


   M508Emu Pic(
      .i_extrst(w_reset),
      .i_extclk(w_iclk),
//    .i_extclk(i_clk),
      .i_rom_dat(w_rom_dat),
      .o_rom_adr(w_rom_adr),
      .o_rom_nread(w_nread),
      .i_rom_valid(w_rom_dat_valid),
      .i_rom_nbusy(w_nbusy),
      .o_led_dat(w_led_dat),
      .o_led_adr(w_led_adr),
      .o_led_we(w_led_we),
      .i_sw(i_sw),
      .o_led_onboard(w_led_obd)
   );



   mLedMatrixDriver LedMatrixDriver(
      .i_clk(w_iclk),
//    .i_clk(i_clk),
      .i_reset(w_reset),
      .i_we(w_led_we),
      .i_adr(w_led_adr),  // i[4:0]
      .i_dat(w_led_dat),  // i[15:0]
      .o_row(w_row),      // o[15:0]
      .o_col(w_col)       // o[31:0]
); 


endmodule
