`timescale 1ns/1ns

module mLedMatrixDriver (
   i_clk,
   i_reset,
   i_we,
   i_adr,
   i_dat,
   o_row,
   o_col
); 
   input i_clk;
   input i_reset;
   input i_we;
   input  [4:0]  i_adr;
   input  [15:0] i_dat;
   output [15:0] o_row;
   output [31:0] o_col;
   
   reg [15:0] r_row;
   reg [31:0] r_col;
   reg [4:0]  r_cnt;
   reg [15:0] r_vram[0:31];   // keep output data (ROW[15:0] x COL[31:0])
   reg [5:0]  r_refresh_cnt;
   reg        r_refresh;

   wire       w_refresh_tim;   
   wire       w_refresh;
   
   
   assign o_row = r_row;
   assign o_col = r_col;

   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_vram[00] <= 16'h0000;
         r_vram[01] <= 16'h0000;
         r_vram[02] <= 16'h0000;
         r_vram[03] <= 16'h0000;
         r_vram[04] <= 16'h0000;
         r_vram[05] <= 16'h0000;
         r_vram[06] <= 16'h0000;
         r_vram[07] <= 16'h0000;
         r_vram[08] <= 16'h0000;
         r_vram[09] <= 16'h0000;
         r_vram[10] <= 16'h0000;
         r_vram[11] <= 16'h0000;
         r_vram[12] <= 16'h0000;
         r_vram[13] <= 16'h0000;
         r_vram[14] <= 16'h0000;
         r_vram[15] <= 16'h0000;
         r_vram[16] <= 16'h0000;
         r_vram[17] <= 16'h0000;
         r_vram[18] <= 16'h0000;
         r_vram[19] <= 16'h0000;
         r_vram[20] <= 16'h0000;
         r_vram[21] <= 16'h0000;
         r_vram[22] <= 16'h0000;
         r_vram[23] <= 16'h0000;
         r_vram[24] <= 16'h0000;
         r_vram[25] <= 16'h0000;
         r_vram[26] <= 16'h0000;
         r_vram[27] <= 16'h0000;
         r_vram[28] <= 16'h0000;
         r_vram[29] <= 16'h0000;
         r_vram[30] <= 16'h0000;
         r_vram[31] <= 16'h0000;
      end
      else if(i_we) begin
         r_vram[i_adr] <= i_dat;
      end
   end


   // Create LED Matrix Update Timing Signal
   // (for Satisfing Transistor Switching Time)
   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_refresh_cnt <= 6'h0;
      end
      else if(w_refresh_tim) begin
         r_refresh_cnt <= 6'h0;
      end
      else begin
         r_refresh_cnt <= r_refresh_cnt + 6'h01;
      end
   end
   
   assign w_refresh_tim = (r_refresh_cnt == 6'd49)? 1'b1: 1'b0;

   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_refresh <= 1'b0;
      end
      else if(w_refresh_tim) begin
         r_refresh <= 1'b1;
      end   
      else begin
         r_refresh <= 1'b0;
      end
   end
   
   assign w_refresh = r_refresh;


   // Column Location Counter(0~31)
   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_cnt <= 5'h00;
      end
      else if(w_refresh) begin
         r_cnt <= r_cnt + 5'h01;
      end
      else begin
         r_cnt <= r_cnt;
      end   
   end

   // control Enabled Column Location
   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_col <= 32'h0000_0000;
      end
      else begin
         case(r_cnt)
            5'h00:   r_col <= 32'h0000_0001;
            5'h01:   r_col <= 32'h0000_0002;
            5'h02:   r_col <= 32'h0000_0004;
            5'h03:   r_col <= 32'h0000_0008;
            5'h04:   r_col <= 32'h0000_0010;
            5'h05:   r_col <= 32'h0000_0020;
            5'h06:   r_col <= 32'h0000_0040;
            5'h07:   r_col <= 32'h0000_0080;
            5'h08:   r_col <= 32'h0000_0100;
            5'h09:   r_col <= 32'h0000_0200;
            5'h0a:   r_col <= 32'h0000_0400;
            5'h0b:   r_col <= 32'h0000_0800;
            5'h0c:   r_col <= 32'h0000_1000;
            5'h0d:   r_col <= 32'h0000_2000;
            5'h0e:   r_col <= 32'h0000_4000;
            5'h0f:   r_col <= 32'h0000_8000;
            5'h10:   r_col <= 32'h0001_0000;
            5'h11:   r_col <= 32'h0002_0000;
            5'h12:   r_col <= 32'h0004_0000;
            5'h13:   r_col <= 32'h0008_0000;
            5'h14:   r_col <= 32'h0010_0000;
            5'h15:   r_col <= 32'h0020_0000;
            5'h16:   r_col <= 32'h0040_0000;
            5'h17:   r_col <= 32'h0080_0000;
            5'h18:   r_col <= 32'h0100_0000;
            5'h19:   r_col <= 32'h0200_0000;
            5'h1a:   r_col <= 32'h0400_0000;
            5'h1b:   r_col <= 32'h0800_0000;
            5'h1c:   r_col <= 32'h1000_0000;
            5'h1d:   r_col <= 32'h2000_0000;
            5'h1e:   r_col <= 32'h4000_0000;
            5'h1f:   r_col <= 32'h8000_0000;
            default: r_col <= 32'h0000_0000;
         endcase
      end
   end

   //Control Output Row Data
   always @(posedge i_reset or posedge i_clk) begin
      if(i_reset) begin
         r_row <= 16'h0000;
      end
      else begin
         r_row <= r_vram[r_cnt];
      end
   end

endmodule

